
#import "../libs/AppleTV.js"

// --------------------------------------------------------------------------------
//      Object: music
//              music is an ATVAApp object for interacting with the
//              Music appliance on Apple TV.
// --------------------------------------------------------------------------------

if (typeof music == 'undefined') {
    music = new ATVAApp("Music");
}

if (typeof music != 'undefined') {
    music._objectName = "music";

    // overriding fromMenu(), fromMenuAndHighlight(), goTo(), and goToAndHighlight() methods from base ATVAApp()

    music.appMenu = function appMenu() {
        if ( ! this._appMenu ) {
            this._appMenu = new ATVAPageStore(this._title);
        }
        return( this._appMenu );
    }

    music.fromMenu = function fromMenu() {
        if ( ! this.appMenu().waitForPage()) {
            UIALogger.logError(this._objectName + ".fromMenu(): failed to waiting for " + this._title);
            return(undefined);
        }
        return(this.appMenu());
    }

    music.fromMenuAndHighlight = function fromMenuAndHighlight(name) {
        var menu = this.fromMenu();
        if (menu) {
            if (menu.moveToTabWithName(name)) {
                // force delay because UI might hangup
                atvaTarget.delay(10);
                return(menu);
            }
            UIALogger.logError(this._objectName + ".fromMenuAndHighlight(): failed to move to " + name + " on " + this._title);
            return(undefined);
        }
        return(undefined);
    }

    music.fromMenuAndSelect = function fromMenuAndSelect(name) {
        var menu = this.fromMenuAndHighlight(name);
        if (menu) {
            menu.selectIntoPage();
            return(true);
        }
        UIALogger.logError(this._objectName + ".fromMenuAndSelect(): failed to select " + name + " on " + this._title);
        return(false);
    }

    music.goTo = function goTo() {
        if (ATVAPageMainMenu.goToAndSelect(this._title)) {
            return(this.fromMenu());
        }
        UIALogger.logError(this._objectName + ".goTo(): failed to get to " + this._title);
        return(undefined);
    }

    music.goToAndHighlight = function goToAndHighlight(name) {
        if ( ! this.goTo()) {
            return(undefined);
        }
        return(this.fromMenuAndHighlight(name));
    }

    music.goToAndSelect = function goToAndSelect(name) {
        if ( ! this.goTo()) {
            return(undefined);
        }
        return(this.fromMenuAndSelect(name));
    }

    music.fromBrowseTopMusicRowColumn = function fromBrowseTopMusicRowColumn(row, column) {
        var topMusic = this.fromMenu()
        if (topMusic) {
            // get off tab bar.
            if (topMusic.isOnTab()) {
                topMusic.selectIntoPage();
            }

            var albumItem = topMusic.moveToNormalItemAt(row, column);
            if ( ! albumItem) {
                UIALogger.logError(this._objectName + ".fromBrowseTopMusicRowColumn(): failed to get to album at row = " + row + " colume = " + column);
                return (undefined);
            }
            var albumTitle = albumItem.name();
            topMusic.selectIntoPage();

            var ap = new ATVAPageList(albumTitle);
            if ( ! ap.waitForPage()) {
                UIALogger.logError(this._objectName + ".fromBrowseTopMusicRowColumn(): failed to load album page for " + albumTitle);
                return (undefined);
            }

            return (ap);
        }
        UIALogger.logError(this._objectName + ".fromBrowseTopMusicRowColumn(): failed to get to Top Music");
        return (undefined);
    }

    music.goToBrowseTopMusicRowColumn = function goToBrowseTopMusicRowColumn(row, column) {
        if ( ! this.goToAndHighlight("Top Music")) {
            UIALogger.logError(this._objectName + ".goToBrowseTopMusicRowColumn(): failed to get to Top Music");
            return (undefined);
        }
        return(this.fromBrowseTopMusicRowColumn(row, column));
    }

    music.fromAlbumSelectTrack = function fromAlbumSelectTrack(title, track) {
        var ap = new ATVAPageList(title);
        if ( ! ap.waitForPage()) {
            UIALogger.logError(this._objectName + ".fromAlbumSelectTrack(): failed to load album page for " + albumTitle);
            return (undefined);
        }

        var trackPattern = new RegExp("^" + track + ".,\\s+(.+)$", "");
        var trackElement = ap.elementForName("", undefined, undefined, undefined, trackPattern);

        if ( ! trackElement) {
            UIALogger.logError(this._objectName + ".fromAlbumSelectTrack(): failed to get find track " + track);
            return (undefined);
        }

        var matchResult = trackElement.name().match(trackPattern);
        ap.moveToItemWithNameAndSelect(trackElement.name());

        var dp = new ATVAPageAudioDetail(matchResult[1]);
        dp._interval = 5;
        dp._timeout = 60;
        if ( ! dp.waitForPage()) {
            UIALogger.logError(this._objectName + ".fromAlbumSelectTrack(): failed to load audio detail page for " + matchResult[1]);
            return (undefined);
        }

        return (dp);
    }
}
